@@ -1,13 +1,16 @@
 #!/usr/bin/env python
 # encoding: utf-8
 
+import json
 from flask import jsonify, request
 
 from blueflask.lib.errors import not_allowed, bad_request
 from blueflask.lib.infos import success
 
 from ...bluebase import Bluebase
+from ... import LOGS
 from ...ocs import OCS
+from ... import REQUEST_DATABASE
 from ..errors import error_handling
 from . import api
 
@@ -16,7 +19,7 @@ BLUEBASE_CLIENT = Bluebase()
 OCS_CLIENT = OCS()
 
 
-@api.route('/clients/authenticate', methods=['POST'])
+@api.route("/clients/authenticate", methods=["POST"])
 @error_handling
 def authenticate():
     """
@@ -128,37 +131,35 @@ def authenticate():
     """
     data = request.get_json()
     auth = request.authorization
-    if data['service_type'] == 'bip':
-        return not_allowed(
-            message='action non autorisée',
-            code='040-05-405'
-            )
+    if data["service_type"] == "bip":
+        return OCS_CLIENT.authentication()
     else:
-        data['request'] = request
+        data["request"] = request
         try:
             result = BLUEBASE_CLIENT.authentication(auth.username, **data)
+            LOGS.logger.info(result)
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+            return bad_request(message="parametres incomplets", code="040-05-400")
         else:
-            if result['status'] == 200:
-                if data['caller_num'].startswith('03900'):
-                    result['data']['num_is_mine'] = 'Vrai'
+            LOGS.logger.info(result)
+            if result["status"] in [200, 204]:
+                if data["caller_num"].startswith("03900"):
+                    result["data"]["num_is_mine"] = "Vrai"
+                LOGS.logger.info(result)
                 response = {
-                    'customer_id': result['data']['customer_id'],
-                    'device_name': result['data'].get('device_name', []),
-                    'last_name': result['data']['last_name'],
-                    'name': result['data']['name'],
-                    'num_is_mine': result['data']['num_is_mine']
-                    }
+                    "code": "040-05-200",
+                    "info": "authentification reussie :)",
+                    "message": result["data"],
+                    "status": 200,
+                }
                 return jsonify(response)
             else:
+                LOGS.logger.info("here we go")
+                LOGS.logger.info(result)
                 return result
 
 
-@api.route('/clients/authenticate/has-code', methods=['POST'])
+@api.route("/clients/authenticate/has-code", methods=["POST"])
 @error_handling
 def has_code():
     """
@@ -202,30 +203,23 @@ def has_code():
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    if data['operator'] != 'bip':
-        return not_allowed(
-            message=(
-                "api seulement accessible via bip"
-                ),
-            code='040-05'
-            )
+    data["request"] = request
+    if data["operator"] != "bip":
+        return not_allowed(message=("action non autorisee"), code="040-05")
     else:
         try:
             result = OCS_CLIENT.has_code(auth.username, **data)
-        except SyntaxError as exc:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+        except SyntaxError:
+            return bad_request(message="parametres incomplets", code="040-05-400")
         else:
-            if result['status'] == 200:
+            if result["status"] == 200:
                 return success()
             else:
+                LOGS.logger.info(result)
                 return result
 
 
-@api.route('/clients/authenticate/verify-code', methods=['POST'])
+@api.route("/clients/authenticate/verify-code", methods=["POST"])
 @error_handling
 def verify_code():
     """
@@ -274,25 +268,23 @@ def verify_code():
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    if data['operator'] != 'bip':
+    data["request"] = request
+    if data["operator"] != "bip":
         return not_allowed("api seulement accessible via bip")
     else:
         try:
             result = OCS_CLIENT.verify_code(auth.username, **data)
-        except SyntaxError as exc:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+        except SyntaxError:
+            return bad_request(message="parametres incomplets", code="040-05-400")
         else:
-            if result['status'] == 200:
+            if result["status"] == 200:
                 return success()
             else:
+                LOGS.logger.info(result)
                 return result
 
 
-@api.route('/clients/authenticate/create-code', methods=['POST'])
+@api.route("/clients/authenticate/create-code", methods=["POST"])
 @error_handling
 def create_code():
     """
@@ -341,25 +333,22 @@ def create_code():
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    if data['operator'] != 'bip':
+    data["request"] = request
+    if data["operator"] != "bip":
         return not_allowed(message="api seulement accessible via bip")
     else:
         try:
             result = OCS_CLIENT.set_code(auth.username, **data)
-        except SyntaxError as exc:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+        except SyntaxError:
+            return bad_request(message="parametres incomplets", code="040-05-400")
         else:
-            if result['status'] == 200:
+            if result["status"] == 200:
                 return success()
             else:
                 return result
 
 
-@api.route('/clients/authenticate/reset-code', methods=['POST'])
+@api.route("/clients/authenticate/reset-code", methods=["POST"])
 @error_handling
 def reset_code():
     """
@@ -414,25 +403,22 @@ def reset_code():
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    data['secret_code'] = data['old_secret_code']
-    del data['old_secret_code']
-    if data['operator'] != 'bip':
+    data["request"] = request
+    data["secret_code"] = data["old_secret_code"]
+    del data["old_secret_code"]
+    if data["operator"] != "bip":
         return not_allowed("api seulement accessible via bip")
     else:
         try:
             result = OCS_CLIENT.verify_code(auth.username, **data)
-        except SyntaxError as exc:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+        except SyntaxError:
+            return bad_request(message="parametres incomplets", code="040-05-400")
         else:
-            if result['status'] == 200:
-                data['secret_code'] = data['new_secret_code']
-                del data['new_secret_code']
+            if result["status"] == 200:
+                data["secret_code"] = data["new_secret_code"]
+                del data["new_secret_code"]
                 response = OCS_CLIENT.set_code(auth.username, **data)
-                if response['status'] == 200:
+                if response["status"] == 200:
                     return success()
                 else:
                     return response
@@ -440,10 +426,7 @@ def reset_code():
                 return result
 
 
-@api.route(
-    '/clients/<string:customer_id>/subscriptions',
-    methods=['GET']
-    )
+@api.route("/clients/<string:customer_id>/subscriptions", methods=["GET"])
 @error_handling
 def get_subscription_infos(customer_id):
     """
@@ -519,60 +502,63 @@ def get_subscription_infos(customer_id):
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    data['customer_id'] = customer_id
-    if data['service_type'] == 'bip':
-        if data['caller_num'] != data['customer_id']:
-            return not_allowed(
-                message='action non autorisée',
-                code='040-05-405'
-                )
+    data["request"] = request
+    data["customer_id"] = customer_id
+    LOGS.logger.info(data)
+    if data["service_type"] == "bip":
+        if data["caller_num"] != data["customer_id"]:
+            return not_allowed(message="action non autorisee", code="040-05-405")
         else:
             try:
                 result = OCS_CLIENT.get_infos(auth.username, **data)
-                if result['status'] in [200, 204]:
+                if result["status"] in [200, 204]:
                     response = {
-                        'customer_id': customer_id,
-                        'subscriptions': result['data'],
-                        'length': len(result['data']),
+                        "code": "040-05-200",
+                        "info": "yay trouvée! :)",
+                        "message": {
+                            "customer_id": customer_id,
+                            "subscriptions": result["data"],
+                            "length": len(result["data"]),
+                        },
+                        "status": "200",
                     }
                     return jsonify(response)
                 else:
                     return result
 
             except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400'
-                    )
+                return bad_request(message="parametres incomplets", code="040-05-400")
     else:
         try:
             result = BLUEBASE_CLIENT.subscriptions(auth.username, **data)
-            if result['status'] in [200, 204]:
-                if result['num_is_mine'] == 'Vrai' or \
-                   data['caller_num'].startswith('03900') or \
-                   data['caller_num'].startswith('3900'):
+            if result["status"] in [200, 204]:
+                if (
+                    result["num_is_mine"] == "Vrai"
+                    or data["caller_num"].startswith("03900")
+                    or data["caller_num"].startswith("3900")
+                ):
                     response = {
-                        'customer_id': customer_id,
-                        'subscriptions': result['data'],
-                        'length': len(result['data']),
+                        "code": "040-05-200",
+                        "info": "yay trouvée! :)",
+                        "message": {
+                            "customer_id": customer_id,
+                            "subscriptions": result["data"],
+                            "length": len(result["data"]),
+                        },
+                        "status": 200,
                     }
                     return jsonify(response)
                 else:
                     return not_allowed(
-                        message="action non autorisée",
-                        code='040-05-405'
+                        message="action non autorisee", code="040-05-405"
                     )
             else:
                 return result
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+            return bad_request(message="parametres incomplets", code="040-05-400")
 
 
-@api.route('/clients/<string:customer_id>/offers', methods=['GET'])
+@api.route("/clients/<string:customer_id>/offers", methods=["GET"])
 @error_handling
 def get_available_offers(customer_id):
     """
@@ -592,7 +578,7 @@ def get_available_offers(customer_id):
       - basicAuth: []
     parameters:
       - name: operator
-        in: bodyc
+        in: body
         required: true
         description: Identifiant de l'opérateur
         schema:
@@ -624,7 +610,7 @@ def get_available_offers(customer_id):
       - name: balance
         in: body
         type: string
-        description: Solde du caller_num si l'utilisateur est extérieur à Bip
+        description: optionnel pour le cas d'un partenaire,
     responses:
       200:
         description: liste des offres
@@ -661,61 +647,41 @@ def get_available_offers(customer_id):
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    data['customer_id'] = customer_id
-
-    if data['operator'] == 'bip':  # on doit chercher le solde de l'utilisateur
+    data["request"] = request
+    # possibiliter de passer dans le headers les parametres
+    mydata = jsonify(request.args)
+    LOGS.logger.info("J'affiche le headers: {}".format(mydata))
+    data["customer_id"] = customer_id
+    if data["service_type"] == "bip":
         try:
-            result = OCS_CLIENT.check_balance(auth.username, **data)
+            result = OCS_CLIENT.get_offers(auth.username, **data)
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
-        else:
-            if result['status'] in [200, 204]:
-                data['balance'] = result['data']['balance']
-            else:
-                return result
-    else:
-        pass
-        # si autre operateur on ne passe pas par ocs
-
-    if data['service_type'] == 'bip':
-        if data['caller_num'] != data['customer_id']:
-            return not_allowed(
-                message='action non autorisée',
-                code='040-05-405'
-                )
-        else:
-            try:
-                result = OCS_CLIENT.get_offers(auth.username, **data)
-            except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400'
-                    )
+            return bad_request(message="parametres incomplets", code="040-05-400")
     else:
         try:
             result = BLUEBASE_CLIENT.offers(auth.username, **data)
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+            return bad_request(message="paramètres incorrecte", code="040-05-400")
 
-    if result['status'] in [200, 204]:
+    if result["status"] in [200, 204]:
         response = {
-            'service_type': data['service_type'],
-            'offers': result['data'],
-            'length': len(result['data'])
-            }
+            "code": "040-05-200",
+            "info": "yay trouvée! :)",
+            "message": {
+                "service_type": data["service_type"],
+                "offers": result["data"],
+                "length": len(result["data"]),
+            },
+            "status": 200,
+        }
         return jsonify(response)
     else:
         return result
 
 
-@api.route('/clients/<string:customer_id>/offers', methods=['POST'])
+# Besoin d'un review :D
+# Somary Maloto
+@api.route("/clients/<string:customer_id>/offers", methods=["POST"])
 @error_handling
 def activate_offer(customer_id):
     """
@@ -785,6 +751,10 @@ def activate_offer(customer_id):
         in: body
         type: string
         description: Solde du caller_num si l'utilisateur est extérieur à Bip
+      - name: partner_ref
+        in: body
+        type: string
+        description: on utilisera ce variable pour la reconciliation
     responses:
       204:
         description: L'achat de l'offre a réussi
@@ -800,67 +770,142 @@ def activate_offer(customer_id):
     """
     data = request.get_json()
     auth = request.authorization
-    data['request'] = request
-    data['customer_id'] = customer_id
-    if data['service_type'] == 'bip':
-        if data['offer_name'] == 'voucher':
+    data["request"] = request
+    data["customer_id"] = customer_id
+    mapping_function = {
+        "voucher": "buy_voucher",
+        "transfer": "transfer",
+        "offer": "buy_offer",
+    }
+    LOGS.logger.info(mapping_function)
+    if data["service_type"] == "bip":
+        if data["offer_name"] == "voucher":
             try:
                 result = OCS_CLIENT.buy_voucher(auth.username, **data)
-                if result['status'] == 200:
+                if result["status"] == 200:
                     response = {
-                        'face_value': result['data']['face_value']
-                        }
+                        "face_value": result["data"]["face_value"],
+                        "balance": result["data"]["balance"],
+                    }
                     return jsonify(response)
                 else:
                     return result
             except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400'
-                    )
-        elif data['offer_name'] == 'transfer':
+                return bad_request(message="parametres incomplets", code="040-05-400")
+        elif data["offer_name"] == "credit_transfer_partner":
+            try:
+                response = OCS_CLIENT.transfer_credit_partner(auth.username, **data)
+                LOGS.logger.info(response)
+                return jsonify(response)
+            except SyntaxError:
+                return bad_request(message="parametres incomplets", code="040-05-400")
+        elif data["offer_name"] == "buy_offer_partner":
+            try:
+                response = OCS_CLIENT.buy_offer_partner(auth.username, **data)
+                LOGS.logger.info(response)
+                return jsonify(response)
+            except SyntaxError:
+                return bad_request(message="parametres incomplets", code="040-05-400")
+        elif data["offer_name"] == "transfer":
             try:
                 result = OCS_CLIENT.transfer(auth.username, **data)
             except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400'
-                    )
-        elif data['offer_name'] == 'offer':
+                return bad_request(message="parametres incomplets", code="040-05-400")
+        elif data["offer_name"] == "offer":
             try:
                 result = OCS_CLIENT.buy_offer(auth.username, **data)
             except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400'
-                    )
+                return bad_request(message="parametres incomplets", code="040-05-400")
     else:
-        if data['operator'] == 'bip':
-            try:
-                result = BLUEBASE_CLIENT.buy_recharge(auth.username, **data)
-                result['token'] = auth.username
-                return jsonify(result)
-            except SyntaxError:
-                return bad_request(
-                    message="paramètres incomplets",
-                    code='040-05-400')
-        # au cas où c'est un partenaire qui fait la requête
-        else:
-            data['balance'] = data.get('balance', 0)
+        data["balance"] = data.get("balance", 0)
         try:
-            result = BLUEBASE_CLIENT.buy_recharge(auth.username, **data)
+            response = BLUEBASE_CLIENT.buy_recharge(auth.username, **data)
+            return jsonify(response)
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
-    if result['status'] in [200, 204]:
-        return success()
+            return bad_request(message="parametres incomplets", code="040-05-400")
+    if result["status"] in [200, 204]:
+        if data["offer_name"] == "voucher":
+            response = {"face_value": result["data"]["face_value"]}
+            return jsonify(response)
+        else:
+            return success()
     else:
         return result
 
 
-@api.route('/clients/<string:phone_number>/locale', methods=['GET'])
+@api.route("/clients/<string:operator>/<string:request_id>", methods=["GET"])
+def request_status(operator, request_id):
+    redis_key = "request:ussd:{}".format(request_id)
+    # TODO traiter le cas où la clé n'existe pas
+    # TODO ecrire une fonction qui permet de marquer dans redis
+    # qu'une requete a été déja vérifié
+    LOGS.logger.info("C'est par ici {}".format(operator))
+    try:
+        result = REQUEST_DATABASE.hgetall(redis_key)
+        LOGS.logger.info(result)
+    except Exception:
+        response = {
+            "info": "une erreur interne s'est produite",
+            "code": "040-05-400",
+            "message": "un souci au niveau de la base",
+            "status": 400,
+        }
+        return jsonify(response)
+    if result:
+        data_partner = {}
+        LOGS.logger.info("ici j'insere les donnees dans data_partner")
+        for key, value in result.items():
+            key = key.decode("utf-8")
+            data = value.decode("utf-8")
+            LOGS.logger.info("{}:{}".format(key, data))
+            # changer le type du champ message en dictionnaire
+            if key == "message":
+                d = data.replace("'", '"')
+                data_partner[key] = json.loads(d)
+                LOGS.logger.info(result)
+                d = data.replace("'", '"')
+                if d:
+                    data_partner[key] = json.loads(d)
+                LOGS.logger.info("nested dictionary {}:{}".format(key, data))
+            else:
+                data_partner[key] = data
+                LOGS.logger.info("activation data: ....".format(data_partner))
+
+        # seul l'opérateur concerné
+        # peut voir le résultat de sa requête
+        if "operator" in data_partner and data_partner["operator"] == operator:
+            response = {
+                "info": "yay transaction trouvee! :)",
+                "code": "040-05-200",
+                "message": data_partner,
+                "status": 200,
+            }
+        elif "operator" in data_partner and data_partner["operator"] != operator:
+            response = {
+                "info": "forbidden",
+                "code": "040-05-405",
+                "message": "action non autorisee",
+                "status": 405,
+            }
+        else:
+            response = {
+                "info": "ressource introuvable",
+                "code": "040-05-404",
+                "message": "Identifiant Introuvable",
+                "status": 404,
+            }
+
+    else:
+        response = {
+            "info": "ressource introuvable",
+            "code": "040-05-404",
+            "message": "Identifiant introuvable",
+            "status": 404,
+        }
+    return jsonify(response)
+
+
+@api.route("/clients/<string:phone_number>/locale", methods=["GET"])
 def locale(phone_number):
     """
     Récupération de la langue utilisée par le client bip.
@@ -917,19 +962,12 @@ def locale(phone_number):
 
     data = request.get_json()
     auth = request.authorization
-    if (data['service_type'] == 'bip' and
-            data['operator'] == 'bip'):
+    if data["service_type"] == "bip" and data["operator"] == "bip":
 
         try:
             result = OCS_CLIENT.get_language(auth.username, **data)
         except SyntaxError:
-            return bad_request(
-                message="paramètres incomplets",
-                code='040-05-400'
-                )
+            return bad_request(message="parametres incomplets", code="040-05-400")
         return result
     else:
-        return bad_request(
-            message="action non autorisee",
-            code='040-05-400'
-            )
+        return bad_request(message="action non autorisee", code="040-05-400")
